/*
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
********************* 504_Programm_Sortierung_aufsteigend *********************
-------------------------------------------------------------------------------
******************************* main.c ** V1.1 ********************************
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
 Sortierung einer 4-stelligen Zeichengruppe in aufsteigender Folge
 Zusaetzliche Header..: keine
 Module.(*.c und *.h).: application, lcd_2wire, num_conversion
 Benoetigte Hardware..: Testboard, 20x4-LCD, 2-Draht-LCD-Interface
 Version..............: 1.1
 Compiler.............: CodeVisionAVR
 Chip.................: ATmega88
 Datum................: Juni 2009
 Autor................: Alwin Lenck (ALE23) basierend auf dem Projekt 
                        2-Draht-LCD von Udo Juerss
-------------------------------------------------------------------------------
Sortierte Anzeige von 4-stelligen Zeichen-Gruppen auf dem 20x4-LCD.
Es werden nicht die Zeichen-Gruppen selbst sortiert, sondern nur die
Pointer auf die Zeichen-Gruppen. Wenn die Zeichen-Gruppen entsprechend der
Pointer-Folge ausgegeben werden, erscheinen sie sortiert auf dem LCD.

Anschluesse des 20x4-LCD:
CLOCK................: PB1 (kann geaendert werden in application.h)
DATA.................: PB2 (kann geaendert werden in application.h)
*/

//-----------------------------------------------------------------------------
// Header-Dateien einfuegen:
//-----------------------------------------------------------------------------
#include "application.h"           // Initialisierung der Anwendung
#include "lcd_2wire.h"             // Initialisierung des 2Draht-LCDs
#include "num_conversion.h"        // Initialisierung der Konvertierung

//-----------------------------------------------------------------------------
// Funktions-Prototypen:
//-----------------------------------------------------------------------------
// int input(char *wort_pointer[]) ist simuliert und entfaellt
void sort(char *p[], int n);       // Sortier-Funktion
void output(char *p[], int n);     // Ausgabe-Funktion

//-----------------------------------------------------------------------------
// Globale Variablen:
//-----------------------------------------------------------------------------
char *wort_pointer[16] =           // Definition eines Arays von 
  {"P404", "N402", "O403", "M401", // 16 Pointers auf char zur 
   "E201", "H204", "G203", "F202", // Simulation einer Eingabe
   "L304", "K303", "J302", "I301", 
   "C103", "B102", "A101", "D104"};
int anz_worte = 16;                // Simulation des Wortzaehlers
int i;

//-----------------------------------------------------------------------------
// Hauptprogramm main
//-----------------------------------------------------------------------------
void main(void)
{
// Die folgenden 2 Anweisungen stammen vom AVR-Projekt 2-Draht-LCD
  // Setzen des System-Taktes (CPU-Clock),
  // Ausschalten nicht benutzter Hardware-Module
  app_init();                      // Initialisierung der Anwendung

  // Definition von Prototypen fuer LCD-Ausgabe-Funktionen usw.
  lcd_init();                      // Initialisierung des LCD-Interfaces
  
  sort(wort_pointer, anz_worte);   // Aufruf der Sortier-Funktion
  output(wort_pointer, anz_worte); // Aufruf der Ausgabe-Funktion
  
  while(1)                         // Endlos-Schleife
    ;                              // ohne weitere Anweisungen
}

//-----------------------------------------------------------------------------
// Sortier-Funktion
//-----------------------------------------------------------------------------
void sort(char *p[], int n)
{
  int a;                           // a => Zaehler fuer aeussere Schleife
  int b;                           // b => Pointer-Zaehler fuer innere Schleife
  char *x;                         // x ist ein Hilfs-Pointer auf char
  for(a = 1; a < n; a++)           // Anz. Durchlaeufe = anz_worte-1 damit
  {                                // jeder String mit jedem verglichen wird.
    for(b = 0; b < n-1; b++)       // Innere Schleife fuer Vergleich: 
    {                              // Beginn mit 0 fuer Array-Index = 0
      if(strcmp(p[b],p[b+1]) > 0)  // Vergleich zweier Strings durch Uebergabe
      {                            // zweier Pointers auf diese Strings.
                                   // Ergebnis > 0 wenn 1. String groesser
                                   // ist als der 2. String. Dann erfolgt ein
        x      = p[b];             // ringfoermiger Austausch der beiden 
        p[b]   = p[b+1];           // Pointers ueber den Hilfs-Pointer x
        p[b+1] = x;                // 
      }                            // Andernfalls: Strings identisch oder
                                   // schon in der richtigen Reihenfolge
    }
  }
}

//-----------------------------------------------------------------------------
// Ausgabe-Funktion
//-----------------------------------------------------------------------------
void output(char *p[], int n)      // Ausgabe-Funktion
{
  int i;                           // Zaehler

  for(i = 0; i < n; i++)
  {
    lcd_goto_xy((i*5)-((i/4)*20),(i/4));  // Spalte: 0-5-10-15
                                          // Zeile: 0-1-2-3
    lcd_write_str(p[i]);           // String mit 4 Zeichen auf LCD ausgeben
  }
}
//-----------------------------------------------------------------------------
